<?php
namespace MSFD;
/**
 * Base Path.
 */
define('BASE_PATH', realpath(__DIR__.'/../'));

// require_once __DIR__.'/Constants.php';
/**
 * Load Composer.
 */
require_once __DIR__.'/../vendor/autoload.php';

/**
 * Init DotEnv.
 */
// $dotEnv = new \Dotenv\Dotenv(BASE_PATH);
// $dotEnv->load();
if (file_exists(BASE_PATH . '/.env')) {
    /**
     * Init DotEnv.
     */
    $dotEnv = new \Dotenv\Dotenv(BASE_PATH);
    $dotEnv->load();
}

class Config {
	const LINE_READ_LIMIT = 500;
	const BYTE_READ_LIMIT = 450000000;
	const MODE = 1;
	// const API_BASE_URL = getenv("https://geoserver.hcmr.gr/geoserver/rest/");
    /* const REPO_API = "http://msfd-repo.ath.hcmr.gr"; */
    const API_URL = array(
        "REPO" => "http://msfd-repo.ath.hcmr.gr/",
        "GSRV" => "https://geoserver.hcmr.gr/geoserver/",
        "POSD" => "https://api.poseidon.hcmr.gr/",
        "WFD" => "https://watermonitoring.hcmr.gr/api/"
    );

	// Database configuration
	// Production / Development
    private static $api_creds;
	// Header translations
	const HEADER_TRANSLATIONS = [
		'Cruise' => 'cruise',
		'Station' => 'station',
		'Type' => 'type',
		'YYYY-MM-DDThh:mm:ss.sss' => 'datetime',
		'YYYY-MM-DDThh:mm' => 'datetime',
		'Longitude [degrees_east]' => 'lon',
		'Latitude [degrees_north]' => 'lat',
		'LOCAL_CDI_ID' => 'cdi',
		'EDMO_code' => 'edmo',
		'Bot. Depth [m]' => 'bottom_depth',
		'DEPHPR01 [m]' => 'depth',
		'DEPHPR01 [meter]' => 'depth',
		'DEPH [m]' => 'depth',
		'COREDIST [m]' => 'depth',
		'DEPHPR01 [m]' => 'depth',
		'MaximumObservationDepth [m]' => 'depth'
	];

	// Sequence according to which the tables are going to be updated through import
	public static function getReplaceableChars() {
		return self::REPLACEABLE_CHARS;
	}


	public static function getLineReadLimit()
	{
		return self::LINE_READ_LIMIT;
	}


	public static function getByteReadLimit()
	{
		return self::BYTE_READ_LIMIT;
	}


	public static function getItemSeparator()
	{
		return self::ITEM_SEPARATOR;
	}

    public static function getAPIBaseURL($api_name)
    {
        $url = getenv("{$api_name}_URL");
        return self::API_URL[$api_name];
    }

	public static function getDatabaseCreds($mode = self::MODE)
	{
        $host = getenv("DB_HOST")
        $name = getenv("DB_NAME")
        $user = getenv("DB_USER")
        $pass = getenv("DB_PASS")

		return array($host, $name, $user, $pass); //self::DB_CREDS['USER'], self::DB_CREDS['PASS']);
	}


    public static function getAPICredentials($api_name)
    {
        $username = getenv("{$api_name}_USER");
        $password = getenv("{$api_name}_PASS");

        return array("username" => $username, "password" => $password);
    }

}

