<?php
namespace MSFD\Controllers;
use MSFD\Services\PoseidonAPI;
use MSFD\Services\WfdAPI;

class InterfaceController {
    public function index() {
        print_r($_GET);
    }

    public function dropdown($data) {
        switch ($data['category']) {
            case 'watergroups':
                require __DIR__.'/../Data/watergroups.php';
                break;
            case 'waterbodies':
                require __DIR__.'/../Data/waterbodies.php';
                break;
            case 'descriptors':
                require __DIR__.'/../Data/descriptors.php';
                break;
            case 'criterias':
                require __DIR__.'/../Data/criterias.php';
                break;
            case 'parameters':
                require __DIR__.'/../Data/parameters.php';
                break;
            default:
                header($_SERVER['SERVER_PROTOCOL'].'404 Not Found');
                break;
        }
    }


    public function geojson($data) {
        $layer = $data['layer'];

        switch ($layer) {
            case 'POSEIDON':
                require __DIR__.'/../Services/PoseidonAPI.php';
                $p = new PoseidonAPI();
                $p->getToken();

                $platforms = $p->getPlatforms()->results;
                $json = json_encode($platforms);
                echo convertToGeoJSON($json, 'lon', 'lat');

                break;
            case 'coastal_wfd':
                require __DIR__.'/../Services/WfdAPI.php';
                $p = new WfdAPI();
                $p->getToken();
                $stations = $p->getStations();

                $json = json_encode($stations);
                echo convertToGeoJSON($json, 'ctStationDecimalLongitude', 'ctStationDecimalLatitude');
                break;
            default:
                $url = "https://portal:@dm!n322@geoserver.hcmr.gr/geoserver/MSFD_NEW/ows?service=WFS&version=1.0.0&request=GetFeature&typeName={$layer}&outputFormat=application/json";
                $readJSONFile = file_get_contents($url);
                echo $readJSONFile;

                break;
        }
    }


    public function charts($data) {
        $category = $data['category'];
        // require __DIR__.'/../data/actions/fetch_charts.php';
        // path to your JSON file
        $file = __DIR__.'/../Data/products/'. $category.'.json';
        // put the content of the file in a variable
        $data = file_get_contents($file);
        // JSON decode
        $obj = json_decode($data);
        // display the name of the first person
        if ($category == 'afppy') {
            $keys = ['FID_', 'YR', 'CENT_LON', 'CENT_LAT', 'TrawlPerc', 'PursesPerc', 'BoatsePerc', 'otherPerc'];

            echo json_encode($obj);
        }
    }


    public function getPartial($input) {
        $data = $_GET;
        $page = $input['partial'];
        $view = new ViewController();
        $view->render($page, $data);
    }


    public function getPage($input) {
        return 0;
    }


    public function getLayerTreeConfig() {
        require __DIR__.'/../Data/actions/fetch_tree_config.php';
        return 0;
    }
}



function convertToGeoJSON($json, $lon_alias, $lat_alias) {
  // decode the JSON string into a PHP object
  $obj = json_decode($json);
/* $POS_PROPERTIES = ['pid', 'tspr', 'platform_code', 'params', 'wmo', 'pi_name', 'dts', 'dte']; */
  // create an array to hold the GeoJSON features
  $features = array();

  // loop through the objects in the JSON and convert them into GeoJSON features
  foreach ($obj as $item) {
    $props = array_keys((array) $item, $POS_PROPERTIES);
    $feature = array(
      'type' => 'Feature',
      'geometry' => array(
        'type' => 'Point',
        'coordinates' => array($item->$lon_alias, $item->$lat_alias)
      ),
      'properties' => $item
    );
    array_push($features, $feature);
  }

  // create the GeoJSON feature collection
  $geoJSON = array(
    'type' => 'FeatureCollection',
    'features' => $features
  );

  // encode the GeoJSON feature collection as a JSON string and return it
  return json_encode($geoJSON);
}
