<?php
namespace MSFD\Services;
use MSFD\Config;


class PgSql
{
    private static $db; //The db handle
    public static $num_rows; //Number of rows
    public static $last_id; //Last insert id
    public static $aff_rows; //Affected rows
    public static $error;


    public static function close()
    {
        if (self::$db) {
            pg_close(self::$db);
            self::$db = null;
        }
    }


    public static function initialize($creds) {
        list($dbhost, $dbname, $dbuser, $dbpass) = $creds;
        self::$db = pg_connect("host={$dbhost} dbname={$dbname} user={$dbuser} password={$dbpass}");
    }

    // For SELECT
    // Returns one row as object
    public static function getRow($sql)
    {
        $result = pg_query(self::$db, $sql);
        $row = pg_fetch_object($result);
        if (pg_last_error()) exit(pg_last_error());
        return $row;
    }

    // For SELECT
    // Returns an array of row objects
    // Gets number of rows
    public static function getRows($sql)
    {
        $result = pg_query(self::$db, $sql);
        if (pg_last_error()) exit(pg_last_error());
        self::$num_rows = pg_num_rows($result);
        $rows = array();
        while ($item = pg_fetch_object($result)) {
            $rows[] = $item;
        }
        return $rows;
    }

    public static function getJson($sql)
    {
        $result = pg_query(self::$db, $sql);
        if (pg_last_error()) exit(pg_last_error());
        self::$num_rows = pg_num_rows($result);
        $rows = array();
        while ($item = pg_fetch_object($result)) {
            $rows[] = $item;
        }
        return json_encode($rows);
    }



    // For SELECT
    // Returns one single column value as a string
    public static function getCol($sql)
    {
        $result = pg_query(self::$db, $sql);
        $col = pg_fetch_result($result, 0);
        if (pg_last_error()) exit(pg_last_error());
        return $col;
    }


    // For SELECT
    // Returns array of all values in one column
    public static function getColValues($sql)
    {
        $result = pg_query(self::$db, $sql);
        $arr = pg_fetch_all_columns($result);
        if (pg_last_error()) exit(pg_last_error());
        return $arr;
    }


    // For INSERT
    // Returns one or more columns of the successful insert
    public static function insert($sql, ...$ret)
    {
        if ($ret != 'NULL') {
            $sql = rtrim($sql, ';');
                $sql .= ' RETURNING ';
                $sql .= implode(',', $ret);
        }
        $result = pg_query(self::$db, $sql);
        if (pg_last_error()) exit(pg_last_error());
            $response = pg_fetch_object($result);
        return json_encode($response);
    }

    // For UPSERT
    // Returns one or more columns of the successful insert
    public static function upsert($sql, $conf, $exc, ...$ret)
    {
      $excluded_columns = implode(',', $exc);
        if ($ret != 'NULL') {
            $sql = rtrim($sql, ';');
            $sql .= "\nON CONFLICT ({$conf}) DO UPDATE
                        SET ";
            $sql .= ' RETURNING ';
            $sql .= implode(',', $ret);
        }
        echo $sql;
        $result = pg_query(self::$db, $sql);
        if (pg_last_error()) exit(pg_last_error());
            $response = pg_fetch_object($result);
        return json_encode($response);
    }

    // For UPDATE, DELETE and CREATE TABLE
    // Returns number of affected rows
    public static function exec($sql)
    {
        $result = pg_query(self::$db, $sql);
        if (pg_last_error()) exit(pg_last_error());
        self::$aff_rows = pg_affected_rows($result);
        return self::$aff_rows;
    }
}

