<?php
namespace MSFD\Services;
use MSFD\Config;

require_once(__DIR__.'/API.php');


class PoseidonAPI extends API {
    private $username;
    private $password;
    private $post_data;
    private $accessToken;
    private $stationList = array("HERAKLION", "MYKON", "ATHOS", "61277", "SARON", "68422");
    private $apiName = 'POSD';


    public function __construct()
    {
        $url = Config::getAPIbaseURL($this->apiName);
        parent::__construct($url);

        $this->username = Config::getAPIusername($this->apiName); 
        $this->password = Config::getAPIpassword($this->apiName); 
    }

    // POST request to get access token
    public function getToken(){
        $endpoint = "o/token/";
        $authStr = base64_encode($this->username.":".$this->password);
        $data = 'grant_type=client_credentials';
        $baseURL = $this->url; // temporarily keep original url in order to use the auth url
        $this->url = "https://auth.poseidon.hcmr.gr/";
        $this->headers["Authorization"] = "Basic {$authStr}";

        $res = $this->post($endpoint, $data, TRUE);
        $this->accessToken = json_decode($res)->access_token;
        $this->url = $baseURL; // return url to the original string
    }

    public function getPlatforms() {
        $endpoint = "api/platforms/";
        $data = array(
            "pid__in" => implode(',', $this->stationList),
            "type" => "MO"
        );

        $this->headers["Authorization"] = "Bearer {$this->accessToken}";
        $this->headers["Accept"] = "application/json";

        return json_decode($this->get($endpoint, $data));
    }

    public function getPlatformData($platform) {
        $endpoint = "api/data/{$platform}/";
        $params = ["ATMS"]; // debugging
        $data = array(
            "param__pname__in" => implode(',', $params),
            "ordering" => 'dt'
        );

        $this->headers["Authorization"] = "Bearer {$this->accessToken}";
        $this->headers["Accept"] = "application/json";

        return json_decode($this->get($endpoint, $data));
    }


    public function getRestrictedFiles($uuid) {
        $this->endpoint = "security/usergroup/users/";
        $this->isPost = false;
    }


    public function getParameterInfo($params) {
        $data = array(
            "pname__in" => $params
        );

        $endpoint = "api/parameters/";
        $this->headers["Authorization"] = "Bearer {$this->accessToken}";
        $this->headers["Accept"] = "application/json";

        return json_decode($this->get($endpoint, $data));
    }


}

