<?php
namespace MSFD\Services;
use MSFD\Config;
require_once(__DIR__.'/API.php');


class WfdAPI extends API {
    private $username;
    private $password;
    private $post_data;
    private $accessToken;
    private $apiName = 'WFD';


    public function __construct()
    {
        $url = Config::getAPIbaseURL($this->apiName);
        parent::__construct($url);

        $this->username = Config::getAPICredentials($this->apiName)->username; 
        $this->password = Config::getAPICredentials($this->apiName)->password; 
        $this->client_id = Config::getAPIclientID($this->apiName);
        $this->client_secret = Config::getAPIclientSecret($this->apiName);
    }

    // POST request to get access token
    public function getToken(){
        $endpoint = "token";
        /* $authStr = base64_encode($this->username.":".$this->password); */
        $data[] = 'grant_type=password';
        $data[] = "client_id=i{$this->client_id}";
        $data[] = "client_secret={$this->client_secret}"; 
        $data[] = "username={$this->username}";
        $data[] = "password={$this->password}";
        /* $this->headers["Accept"] = "x-www-form-urlencoded"; */
        $data = implode("&", $data);
        $this->headers["Content-Type"] = "application/x-www-form-urlencoded";
        $baseURL = $this->url; // temporarily keep original url in order to use the auth url
        $this->url = "https://sso-iw.hcmr.gr/auth/realms/imbriw/protocol/openid-connect/";

        /* $this->headers["Authorization"] = "Basic {$authStr}"; */

        $res = $this->post($endpoint, $data);
        $this->accessToken = json_decode($res)->access_token;
        $this->url = $baseURL; // return url to the original string
    }

    public function getStations() {
        $endpoint = "coastal-trans-stations?ctStationHabitat.equals=Coastal";

        /* $test = http_build_query($data); */
        $this->headers["Authorization"] = "Bearer {$this->accessToken}";
        $this->headers["Accept"] = "application/json";

        return json_decode($this->get($endpoint, $data));
    }


    public function getStationSamples($station_id) {
        $endpoint = "coastal-trans-samples";

        $this->headers["Authorization"] = "Bearer {$this->accessToken}";
        $this->headers["Accept"] = "application/json";
        $data = array(
            "coastalTransStationId.equals" => $station_id
        );
        $res = json_decode($this->get($endpoint, $data));

        return $res;
    }

    /**
    * Fetches data for a specific station according to the sample sampleIds
    *
     */
    public function getData($parameter_ids, $data_category='all') {

        $this->headers["Authorization"] = "Bearer {$this->accessToken}";
        $this->headers["Accept"] = "application/json";
        $res = [];
        $data = array(
            "coastalTransSampleId.in" => implode(",", $parameter_ids)
        );

        if ($data_category === 'all')
            $endpoints = array("coastal-trans-abiotics","coastal-trans-macrofaunas", "coastal-trans-phytoplanktons", "coastal-trans-angiosperms");

        foreach ($endpoints as $endpoint) {
            array_push($res, json_decode($this->get($endpoint, $data)));
        }

        return $res;
    }

}

